require 'spec_helper'

describe RSpec::XSD::Matcher do
  let(:path)        { File.join(File.expand_path(File.dirname(__FILE__)), '..', '..', '..', 'fixtures') }
  let(:schema_path) { File.join(path, 'schema.xsd') }

  let(:matcher) { RSpec::XSD::Matcher.new(schema_path, File.basename(schema_path)) }

  describe '#matches?' do
    let(:xml) { File.read(File.join(path, 'valid.xml')) }

    context 'with valid XML' do
      it 'returns true' do
        expect(matcher.matches?(xml)).to be(true)
      end
    end

    context 'with XML object' do
      it 'returns true' do
        expect(matcher.matches?(Nokogiri::XML(xml))).to be(true)
      end
    end

    context 'with invalid XML' do
      let(:xml) { File.read(File.join(path, 'invalid.xml')) }

      it 'returns false' do
        expect(matcher.matches?(xml)).to be(false)
      end
    end

    context 'with schema object' do
      it 'returns true' do
        schema = Nokogiri::XML::Schema(File.read(schema_path))
        expect(RSpec::XSD::Matcher.new(schema, File.basename(schema_path)).matches?(xml)).to be(true)
      end
    end
  end

  describe '#description' do
    it 'returns description' do
      expect(matcher.description).to eq('validate against schema.xsd')
    end
  end

  describe '#failure_message' do
    let(:xml) { File.read(File.join(path, 'invalid.xml')) }

    before do
      matcher.matches?(xml)
    end

    it 'returns message with validation errors' do
      expect(matcher.failure_message).to match(/expected that XML would validate against schema.xsd\r\n\r\n/)
    end
  end

  describe '#failure_message_when_negated' do
    it 'returns message' do
      expect(matcher.failure_message_when_negated).to eq('expected that XML would not validate against schema.xsd')
    end
  end
end
